/*******************************************************************
 * Xatm -- OCF, CID, TrueType scalable font support for X 
 *  By:  ͵ <yokota@netlab.is.tsukiba.ac.jp>
 */

/* gs.c -- ghostscript  */
/* $Id: gs.c,v 1.9 1999/03/26 00:47:11 elca Exp $ */

#include <unistd.h>
#include <sys/stat.h>
#include "Xatm.h"

/*#define dprintf(str) fprintf str*/
#define dprintf(str) /* */

static pid_t gsPid = 0;
static int gs[2][2]; /* ghostscript Ȥ̿
			[0][1] xfs ---> gs [0][0]
			[1][0] xfs <--- gs [1][1]
			*/
static int gsWaitSec = GS_WAIT_SEC;


/* ghostscript εưȽ */
void gsInit(void)
{
	dprintf((stdout, "gsinit\n"));

	/* ghostscript Ȥ̿ */
	if(pipe(gs[0]) != 0) {
		printf("gs[0] pipe error\n");
		exit(1);
	}
	if(pipe(gs[1]) != 0) {
		printf("gs[1] pipe error\n");
		exit(1);
	}
	switch(gsPid = fork()) {
	case -1:
		printf("fork gs error\n");
		exit(1);
		break;
	case 0:
		/* gs -> xfs ؤ̿ */
		close(STDIN_FILENO);
		dup(gs[0][0]);
		close(STDOUT_FILENO);
		dup(gs[1][1]);

		close(gs[0][0]);
		close(gs[0][1]);
		close(gs[1][0]);
		close(gs[1][1]);

		/* ghostscript ư */
		/* gs -dSAFER -sDEVICE=bit -sOutputFile=- -q -dNOPAUSE
		    -c "/.setpagesize { //systemdict /statusdict get begin .setpagesize end } bind def"
		    - */
		execlp("gs", "gs", "-dSAFER", "-sDEVICE=bit", "-sOutputFile=-",
		       "-q", "-dNOPAUSE",
		       "-c",
		       "/.setpagesize { //systemdict /statusdict get begin .setpagesize end } bind def",
		       "-", NULL);
		printf("execlp gs error\n");
		exit(1);
		break;
	default:
		/* xfs -> gs ؤ̿ʳϺ */
		close(gs[0][0]);
		close(gs[1][1]);
		break;
	}

	xatm_nonblock(gs[1][0]);
}

/* ghostscript λ */
void gsDeinit(void)
{
	if(gsPid == 0) {
		/*printf(__FUNCTION__ " pid==0\n");*/
		return; /* Ǥ ghostscript ϽλƤ */
	}

	xatm_flush(gs[1][0]);

	dprintf((stdout, "gsquit\n"));

	write(gs[0][1], "quit\n", strlen("quit\n"));

	/* ̿Ĥ */
	close(gs[0][1]);
	close(gs[1][0]);
}

/* ghostscript ³줿 եǥץ֤ */
GsPipe gsGetPipe(void)
{
	GsPipe ret;

	dprintf((stdout, __FUNCTION__ "\n"));


	if(gsPid == 0) { /* ghostscript ưƤʤ */
		gsInit();
		ret.init = XATM_TRUE;
	} else {
		ret.init = XATM_FALSE;
	}

	dprintf((stdout, __FUNCTION__ " pid: %d\n", gsPid));

	ret.r = gs[1][0];
	ret.w = gs[0][1];

	return ret;
}

inline pid_t gsGetPid(void)
{
	return gsPid;
}

inline void gsSetWaitSec(int ival)
{
	gsWaitSec = ival;
}

inline int gsGetWaitSec(void)
{
	/*printf(__FUNCTION__ ": %d\n", gsWaitSec);*/
	return gsWaitSec;
}

/* SIGALRM ϥɥ */
void gsAlarmHandler(int signum)
{
	/*printf(__FUNCTION__ "\n");*/

	alarm(0); /* ޡ */

	gsDeinit();
}

/* SIGCHLD ϥɥ */
void gsChildHandler(int signum)
{
	/*printf(__FUNCTION__ "\n");*/

	if(wait(NULL) == gsPid) {
		gsPid = 0;
	}
}

/* end */
