/* */

#ifndef _UTIL_H
#define _UTIL_H

#include <fontmisc.h>
#include <fontstruct.h>
#include <FSproto.h>

/* atom.c */
Atom MakeAtom(char *string, unsigned len, int makeit);
int ValidAtom(Atom atom);
char *NameForAtom(Atom atom);

/* fontaccel.c */
int FontComputeInfoAccelerators(FontInfoPtr pFontInfo);
int FontCouldBeTerminal(FontInfoPtr pFontInfo);

/* fontnames.c */
void FreeFontNames(FontNamesPtr pFN);
FontNamesPtr MakeFontNamesRecord(unsigned int size);
int AddFontNamesName(FontNamesPtr names, char *name, int length);

/* fontutil.c */
void GetGlyphs(FontPtr     font,
	       unsigned long count,
	       unsigned char *chars,
	       FontEncoding fontEncoding,
	       unsigned long *glyphcount,	/* RETURN */
	       CharInfoPtr *glyphs);	/* RETURN */
void QueryGlyphExtents(FontPtr pFont,
		       CharInfoPtr *charinfo,
		       unsigned long count,
		       ExtentInfoRec *info);
Bool QueryTextExtents(FontPtr pFont,
		      unsigned long count,
		      unsigned char *chars,
		      ExtentInfoRec *info);
Bool ParseGlyphCachingMode(char *str);
void InitGlyphCaching(void);
void SetGlyphCachingMode(int newmode);
int add_range(fsRange *newrange,
	      int *nranges,
	      fsRange **range,
	      Bool charset_subset);

/* fontxlfd.c */
#include	<fontxlfd.h>
#include	<X11/Xos.h>
#include	<math.h>
#include	<stdlib.h>
#include	<locale.h>
#include	<ctype.h>
#include	<stdio.h>	/* for sprintf() */
double xlfd_round_double(double x);
Bool FontParseXLFDName(char       *fname,
		       FontScalablePtr vals,
		       int         subst);
fsRange *FontParseRanges(char *name,
			 int *nranges);

/* format.c */
#include	<FSproto.h>
#include	<font.h>
int CheckFSFormat(fsBitmapFormat format,
		  fsBitmapFormatMask fmask,
		  int        *bit_order,
		  int *byte_order,
		  int *scan,
		  int *glyph,
		  int *image);

/* miscutil.c */
#include <X11/Xosdefs.h>
#include <stdlib.h>
unsigned long *Xalloc(int m);
unsigned long *Xrealloc(unsigned long *n, int m);
void Xfree(unsigned long *n);
void CopyISOLatin1Lowered(char *dst, char *src, int len);
void register_fpe_functions(void);

/* patcache.c */
#include    <fontmisc.h>
#include    <fontstruct.h>
FontPatternCachePtr MakeFontPatternCache(void);
void FreeFontPatternCache(FontPatternCachePtr cache);
void EmptyFontPatternCache(FontPatternCachePtr cache);
void CacheFontPattern(FontPatternCachePtr cache,
		      char *pattern,
		      int patlen,
		      FontPtr pFont);
FontPtr FindCachedFontPattern(FontPatternCachePtr cache,
			      char *pattern,
			      int patlen);
void RemoveCachedFontPattern(FontPatternCachePtr cache,
			     FontPtr		pFont);

/* private.c */
#include    <fontmisc.h>
#include    <fontstruct.h>
int AllocateFontPrivateIndex(void);
void ResetFontPrivateIndex(void);
Bool _FontSetNewPrivate(FontPtr pFont,
			int	    n,
			pointer ptr);

/* utilbitmap.c */
void BitOrderInvert(register unsigned char *buf,
		    register int nbytes);
void TwoByteSwap(register unsigned char *buf,
		 register int nbytes);
void FourByteSwap(register unsigned char *buf,
		  register int nbytes);
int RepadBitmap(char *pSrc, char *pDst,
		unsigned int srcPad, unsigned int dstPad,
		int width, int height);

#endif _UTIL_H
